precision highp float;

// Attributes for WebGL 2.0
//in vec3 position;
//in vec3 normal;
//in vec4 tangent;
//in vec2 uv;
//in vec2 uv2;
// Attributes for WebGL 1.0
attribute vec3 position;
attribute vec3 normal;
attribute vec4 tangent;
attribute vec2 uv;
#ifdef USE_UV2
	attribute vec2 uv2;
#endif

uniform vec4 u_mainuvtransform;
uniform vec4 u_detailuvtransform;
uniform mat4 u_normalMatrix;
uniform mat4 u_tangentMatrix;
uniform mat4 u_inverseview;
#include<instancesDeclaration> //defines uniform mat4 world and other important instance related uniforms
uniform mat4 viewProjection;
uniform mat4 view;

uniform mat4 directionalShadowLightMat;

uniform mat4 spotLightShadowLightMat;
uniform vec2 depthValues;

uniform mat4 KeyLightShadowLightMat;
uniform vec2 keyDepthValues;
uniform mat4 FillLightShadowLightMat;
uniform vec2 fillDepthValues;
uniform mat4 BackLightShadowLightMat;
uniform vec2 backDepthValues;

// Output variables for WebGL 2.0
//out vec3 v_position;
//out vec4 v_uv;
//out vec3 v_normal;
//out vec3 v_tangentuws;
//out vec3 v_tangentvws;
// Output variables for WebGL 1.0
varying vec3 v_position;
varying vec4 v_uv;
varying vec3 v_normal;
varying vec3 v_tangentuws;
varying vec3 v_tangentvws;
varying vec3 v_view;

/// DIRECTIONAL LIGHT VARYING
varying vec4 v_PositionFromDirLight;

/// SPOT LIGHT VARYINGS
varying vec4 v_PositionFromSpotLight;
varying float v_DepthMetric;

/// THREE POINT LIGHT VARYINGS
varying vec4 v_PositionFromKeyLight;
varying float v_KeyLightDepthMetric;
varying vec4 v_PositionFromBackLight;
varying float v_BackLightDepthMetric;
varying vec4 v_PositionFromFillLight;
varying float v_FillLightDepthMetric;

void main() {
	#include<instancesVertex> // uses world or world# (when instances are used) matrices to create finalWorld for use in the rest of the vertex function

		vec4 worldPos = finalWorld * vec4(position, 1.0);
		v_position = vec3(worldPos);

		gl_Position = viewProjection * worldPos;
	#ifdef DEPTHTEST_ONTOP
		gl_Position.z = 0.0;
	#endif
		
	#if defined(INSTANCES)
		mat3 normalWorld = mat3(finalWorld);
		v_normal = normal / vec3(dot(normalWorld[0], normalWorld[0]), dot(normalWorld[1], normalWorld[1]), dot(normalWorld[2], normalWorld[2]));
		v_normal = normalize(normalWorld * v_normal);
	#else
		v_normal = normalize((mat3(u_normalMatrix) * normal).xyz);
	#endif

		v_tangentuws = normalize(u_tangentMatrix * vec4(tangent.xyz,0.0)).xyz;
		v_tangentvws = cross(v_normal, v_tangentuws) * tangent.w;

		v_view = u_inverseview[3].xyz;

		v_uv.xy = uv * u_mainuvtransform.xy + u_mainuvtransform.zw;
	#ifdef USE_UV2
		v_uv.zw = uv2 * u_detailuvtransform.xy + u_detailuvtransform.zw;
	#else
		v_uv.zw = uv * u_detailuvtransform.xy + u_detailuvtransform.zw;
	#endif

	#if defined(DIRECTIONAL_LIGHTING)
		v_PositionFromDirLight = directionalShadowLightMat * finalWorld * vec4(position, 1.0);
	#endif 

	#if defined(SPOT_LIGHTING)
		v_PositionFromSpotLight = spotLightShadowLightMat * finalWorld * vec4(position, 1.0);
		v_DepthMetric = ((v_PositionFromSpotLight.z + depthValues.x) / (depthValues.y));
	#endif

	#if defined(THREEPOINTLIGHTING)
		v_PositionFromKeyLight = KeyLightShadowLightMat * finalWorld * vec4(position, 1.0);
		v_KeyLightDepthMetric = ((v_PositionFromKeyLight.z + keyDepthValues.x) / (keyDepthValues.y));
		v_PositionFromBackLight = BackLightShadowLightMat * finalWorld * vec4(position, 1.0);
		v_BackLightDepthMetric = ((v_PositionFromBackLight.z + backDepthValues.x) / (backDepthValues.y));
		v_PositionFromFillLight = FillLightShadowLightMat * finalWorld * vec4(position, 1.0);
		v_FillLightDepthMetric = ((v_PositionFromFillLight.z + fillDepthValues.x) / (fillDepthValues.y));
	#endif
}
